@@ -112,7 +112,7 @@ _get_table  (hb_tag_t tag, void *user_data)
 
   return hb_blob_create ((const char *) buffer, length,
 			 HB_MEMORY_MODE_WRITABLE,
-			 g_free, NULL);
+			 g_free, buffer);
 }
 
 
@@ -137,7 +137,7 @@ pango_ot_info_get (FT_Face face)
   else
     {
       if (face->generic.finalizer)
-        face->generic.finalizer (face->generic.data);
+        face->generic.finalizer (face);
 
       info = face->generic.data = g_object_new (PANGO_TYPE_OT_INFO, NULL);
       face->generic.finalizer = pango_ot_info_finalizer;
@@ -301,8 +301,7 @@ synthesize_class_def (PangoOTInfo *info)
 
   g_array_free (glyph_infos, TRUE);
 
-  hb_ot_layout_build_glyph_classes (info->hb_face, info->face->num_glyphs,
-				    glyph_indices, classes, j);
+  hb_ot_layout_build_glyph_classes (info->hb_face, glyph_indices, classes, j);
 
   g_free (glyph_indices);
   g_free (classes);
@@ -554,6 +553,7 @@ _pango_ot_info_substitute  (const PangoOTInfo    *info,
 					       &lookup_count,
 					       lookup_indexes);
 
+      lookup_count = MIN (G_N_ELEMENTS (lookup_indexes), lookup_count);
       for (j = 0; j < lookup_count; j++)
 	hb_ot_layout_substitute_lookup (info->hb_face,
 					buffer->buffer,
@@ -601,6 +601,7 @@ _pango_ot_info_position    (const PangoOTInfo    *info,
 					       &lookup_count,
 					       lookup_indexes);
 
+      lookup_count = MIN (G_N_ELEMENTS (lookup_indexes), lookup_count);
       for (j = 0; j < lookup_count; j++)
 	hb_ot_layout_position_lookup (info->hb_face, hb_font,
 				      buffer->buffer,
